import argparse
import time
from multiprocessing import Process

from overcooked_simulator.utils import (
    url_and_port_arguments,
    disable_websocket_logging_arguments,
    add_list_of_manager_ids_arguments,
)

USE_STUDY_SERVER = True


def start_game_server(cli_args):
    from overcooked_simulator.game_server import main

    main(cli_args.url, cli_args.port, cli_args.manager_ids)


def start_study_server(cli_args):
    from overcooked_simulator.example_study_server import main

    main(
        cli_args.url,
        cli_args.port,
        game_server_url_="localhost:8000",
        manager_ids=cli_args.manager_ids,
    )


def start_pygame_gui(cli_args):
    from overcooked_simulator.gui_2d_vis.overcooked_gui import main

    main(
        cli_args.url,
        cli_args.port,
        cli_args.manager_ids,
        CONNECT_WITH_STUDY_SERVER=USE_STUDY_SERVER,
    )


def main(cli_args=None):
    study_server = None
    study_server_args = None
    if USE_STUDY_SERVER:
        parser = argparse.ArgumentParser(
            prog="Overcooked Simulator Study Server",
            description="Study Server: Match Making, client pre and post managing.",
            epilog="For further information, see https://scs.pages.ub.uni-bielefeld.de/cocosy/overcooked-simulator/overcooked_simulator.html",
        )
        url_and_port_arguments(
            parser=parser, server_name="Study Server", default_port=8080
        )
        add_list_of_manager_ids_arguments(parser=parser)
        study_server_args = parser.parse_args()

    if cli_args is None:
        parser = argparse.ArgumentParser(
            prog="Overcooked Simulator",
            description="Game Engine Server + PyGameGUI: Starts overcooked game engine server and a PyGame 2D Visualization window in two processes.",
            epilog="For further information, see https://scs.pages.ub.uni-bielefeld.de/cocosy/overcooked-simulator/overcooked_simulator.html",
        )
        url_and_port_arguments(parser)
        disable_websocket_logging_arguments(parser)
        add_list_of_manager_ids_arguments(parser)

        cli_args = parser.parse_args()

    game_server = None
    pygame_gui = None
    try:
        print("Start game engine:")
        game_server = Process(target=start_game_server, args=(cli_args,))
        game_server.start()
        time.sleep(0.5)

        if USE_STUDY_SERVER:
            print("Start study server:")
            study_server = Process(target=start_study_server, args=(study_server_args,))
            study_server.start()
            time.sleep(0.5)

        print("Start PyGame GUI:")
        pygame_gui = Process(target=start_pygame_gui, args=(cli_args,))
        pygame_gui.start()

        if USE_STUDY_SERVER:
            print("Start PyGame GUI:")
            pygame_gui_2 = Process(target=start_pygame_gui, args=(cli_args,))
            pygame_gui_2.start()

            print("Start PyGame GUI:")
            pygame_gui_3 = Process(target=start_pygame_gui, args=(cli_args,))
            pygame_gui_3.start()
            while (
                pygame_gui.is_alive()
                and pygame_gui_2.is_alive()
                and pygame_gui_3.is_alive()
            ):
                time.sleep(1)

        else:
            while pygame_gui.is_alive():
                time.sleep(1)

    except KeyboardInterrupt:
        print("Received Keyboard interrupt")
    finally:
        if USE_STUDY_SERVER and study_server is not None and study_server.is_alive():
            print("Terminate study server")
            study_server.terminate()
        if game_server is not None and game_server.is_alive():
            print("Terminate game server")
            game_server.terminate()
        if pygame_gui is not None and pygame_gui.is_alive():
            print("Terminate pygame gui")
            pygame_gui.terminate()
        time.sleep(0.1)


if __name__ == "__main__":
    main()
