from datetime import timedelta

from overcooked_simulator.overcooked_environment import Environment


class InfoMsgManager:
    def __init__(
        self,
        name: str,
        env: Environment,
        msg: str,
        duration: int = 5,
        level: str = "Normal",
    ):
        self.msg = msg
        self.duration = timedelta(seconds=duration)
        self.level = level

    def __call__(self, hook_ref: str, env: Environment, **kwargs):
        for player_id in env.players:
            env.info_msgs_per_player[player_id].append(
                {
                    "msg": self.msg,
                    "start_time": env.env_time,
                    "end_time": env.env_time + self.duration,
                    "level": self.level,
                }
            )
        self.remove_old_msgs(env)

    @staticmethod
    def remove_old_msgs(env: Environment):
        for player_id, msgs in env.info_msgs_per_player.items():
            delete_msgs = []
            for idx, msg in enumerate(msgs):
                if msg["end_time"] < env.env_time:
                    delete_msgs.append(idx)
            for idx in reversed(delete_msgs):
                msgs.pop(idx)
